import os
import sysconfig
import multiprocessing
from pathlib import Path
from typing import List

from setuptools import Extension, Distribution

from Cython.Build import cythonize
from Cython.Distutils.build_ext import new_build_ext as cython_build_ext

SOURCE_DIR = Path("src")
BUILD_DIR = Path("dist")

class BuildExtWithoutPlatformSuffix(cython_build_ext):
    def get_ext_filename(self, ext_name):
        filename = super().get_ext_filename(ext_name)
        return self.get_ext_filename_without_platform_suffix(filename)
    
    def get_ext_filename_without_platform_suffix(self, filename):
        
        name, ext = os.path.splitext(filename)
        ext_suffix = sysconfig.get_config_var('EXT_SUFFIX')

        if ext_suffix == ext:
            return filename

        ext_suffix = ext_suffix.replace(ext, '')
        idx = name.find(ext_suffix)

        if idx == -1:
            return filename
        else:
            return name[:idx] + ext

def build() -> None:
    # Collect and cythonize all files
    extension_modules = cythonize_helper(get_extension_modules())

    # Use Setuptools to collect files
    distribution = Distribution(
        {
            "ext_modules": extension_modules,
            "cmdclass": {
                "build_ext": BuildExtWithoutPlatformSuffix,
            },
        }
    )

    # Grab the build_ext command and copy all files back to source dir.
    # Done so Poetry grabs the files during the next step in its build.
    distribution.run_command("build_ext")
    build_ext_cmd = distribution.get_command_obj("build_ext")
    build_ext_cmd.copy_extensions_to_source()
    # Set the value to 1 for "inplace", with the goal to build extensions
    # in build directory, and then copy all files back to the source dir
    # (under the hood, "copy_extensions_to_source" will be called after
    # building the extensions). This is done so Poetry grabs the files
    # during the next step in its build.
    # build_ext_cmd.ensure_finalized()
    # build_ext_cmd.inplace = 1
    # build_ext_cmd.run()


def get_extension_modules() -> List[Extension]:
    """Collect all .py files and turn them into Distutils/Setuptools
    Extensions"""

    extension_modules: List[Extension] = []

    for py_file in SOURCE_DIR.rglob("*.py"):
        # Get path (not just name) without .py extension
        module_path = py_file.with_suffix("")

        # Convert path to module name
        module_path = module_path.as_posix().replace("/", ".")

        extension_module = Extension(
            name=module_path,
            sources=[str(py_file)]
        )

        extension_modules.append(extension_module)

    return extension_modules


def cythonize_helper(extension_modules: List[Extension]) -> List[Extension]:
    """Cythonize all Python extensions"""

    return cythonize(
        module_list=extension_modules,

        # Don't build in source tree (this leaves behind .c files)
        build_dir=BUILD_DIR,

        # Don't generate an .html output file. This will contain source.
        annotate=False,

        # Parallelize our build
        nthreads=multiprocessing.cpu_count() * 2,

        # Tell Cython we're using Python 3
        compiler_directives={"language_level": "3"},

        # (Optional) Always rebuild, even if files untouched
        force=True,
    )


if __name__ == '__main__':
    build()





# import os

# try:
#     from Cython.Build import cythonize
# except ImportError:
#     def build(setup_kwargs):
#         pass
# else:
#     from setuptools import Extension
#     from setuptools.dist import Distribution
#     from distutils.command.build_ext import build_ext
    
#     def get_export_symbols_fixed(self, ext):
#         pass
    
#     build_ext.get_export_symbols = get_export_symbols_fixed
    
#     def build(setup_kwargs):
#         extensions = [
#             'src/CosmosCore/*.py',
#             'src/CosmosCore/Application/*.py',
#             'src/CosmosCore/Application/console/*.py'
#         ]
#         includePaths = [
            
#         ]
#         # gcc args hack : enable optimizations
#         os.environ['CFLAGS'] = '-03'
        
#         setup_kwargs.update({
#             'ext_modules': cythonize(
#                 extensions,
#                 language_level = 3,
#                 compiler_directives = {'linetrace' : True},
#                 include_path = includePaths
#             ),
#             # 'cmdclass': {'build.ext' : build_ext}
#         })